Module reshape

Module reshape 

The reshape array to array codec (Experimental).

Performs a reshaping operation.

This codec is experimental and may be incompatible with other Zarr V3 implementations.

Compatible Implementations

None

Specification

Codec name Aliases (Zarr V3)

  • reshape

Codec id Aliases (Zarr V2)

None

Codec configuration Example - ReshapeCodecConfiguration:

{
    "shape": [[0, 1], -1, [3], 10]
}

Structs

ReshapeCodec
A reshape codec implementation.
ReshapeCodecConfigurationV1
reshape codec configuration parameters (version 1.0).
ReshapeShape
An array specifying the size B_shape[i] of each dimension i of the output array B as a function of the shape A_shape of the input array A.

Enums

ReshapeCodecConfiguration
A wrapper to handle various versions of Reshape codec configuration parameters.
ReshapeDim
An element of ReshapeShape.