pub struct ReshapeCodecConfigurationV1 {
pub shape: ReshapeShape,
}Expand description
reshape codec configuration parameters (version 1.0).
Fields
shape: ReshapeShapeThe reshape order defining how to permute the array.
Implementations
Sourceimpl ReshapeCodecConfigurationV1
impl ReshapeCodecConfigurationV1
Sourcepub const fn new(shape: ReshapeShape) -> ReshapeCodecConfigurationV1
pub const fn new(shape: ReshapeShape) -> ReshapeCodecConfigurationV1
Create a new Reshape codec configuration given a ReshapeShape.