Module pcodec

Module pcodec 

Available on crate feature pcodec only.
The pcodec array to bytes codec (Experimental).

This codec is experimental and may be incompatible with other Zarr V3 implementations.

Pcodec (or Pco, pronounced “pico”) losslessly compresses and decompresses numerical sequences with high compression ratio and fast speed.

This codec requires the pcodec feature, which is disabled by default.

Compatible Implementations:

This codec is fully compatible with the numcodecs.pcodec codec in zarr-python.

Specification

Codec name Aliases (Zarr V3)

  • numcodecs.pcodec
  • https://codec.zarrs.dev/array_to_bytes/pcodec

Codec id Aliases (Zarr V2)

  • pcodec

Codec configuration Example - PcodecCodecConfiguration:

{
    "level": 5,
    "mode_spec": "auto",
    "delta_spec": "auto",
    "paging_spec": "equal_pages_up_to",
    "delta_encoding_order": null,
    "equal_pages_up_to": 262144
}

Structs

PcodecCodec
A pcodec codec implementation.
PcodecCodecConfigurationV1
pcodec codec configuration parameters (version 1.0 draft).
PcodecCompressionLevel
An integer from 0 to 12 controlling the compression level.
PcodecDataTypePlugin
Plugin for registering pcodec codec support for a data type.
PcodecDeltaEncodingOrder
An integer from 0 to 7 controlling the delta encoding order.

Enums

PcodecCodecConfiguration
A wrapper to handle various versions of pcodec codec configuration parameters.
PcodecElementType
The pcodec element type for dispatching to the pcodec library.

Traits

PcodecDataTypeExt
Extension trait for DataType to access PcodecDataTypeTraits.
PcodecDataTypeTraits
Traits for a data type supporting the pcodec codec.