Available on crate feature
sharding only.Expand description
The sharding array->bytes codec.
Sharding logically splits chunks (shards) into sub-chunks (inner chunks) that can be individually compressed and accessed. This allows to colocate multiple chunks within one storage object, bundling them in shards.
See https://zarr-specs.readthedocs.io/en/latest/v3/codecs/sharding-indexed/v1.0.html.
This codec requires the sharding feature, which is enabled by default.
See ShardingCodecConfigurationV1 for example JSON metadata.
The ShardingCodecBuilder can help with creating a ShardingCodec.
Structs
- A Sharding codec implementation.
- A
ShardingCodecbuilder. - Sharding codec configuration parameters.
Enums
- A wrapper to handle various versions of Sharding codec configuration parameters.
- The sharding index location.