Enum zarrs::array::codec::array_to_bytes::sharding::ShardingIndexLocation
source · pub enum ShardingIndexLocation {
Start,
End,
}Available on crate feature
sharding only.Expand description
The sharding index location.
Variants
Start
The index is at the start of the shard, before the chunks.
End
The index is at the end of the shard, after the chunks.