Module zfpy

Module zfpy 

Available on crate feature zfp only.
The zfpy array to bytes codec (Experimental).

This codec is experimental and may be incompatible with other Zarr V3 implementations.

zfp is a compressed number format for 1D to 4D arrays of 32/64-bit floating point or integer data. 8/16-bit integer types are supported through promotion to 32-bit in accordance with the zfp utility functions.

This codec requires the zfp feature, which is disabled by default.

Compatible Implementations

This codec is fully compatible with the numcodecs.zfpy codec in zarr-python.

Specification

Codec name Aliases (Zarr V3)

  • numcodecs.zfpy
  • https://codec.zarrs.dev/array_to_bytes/zfpy

Codec id Aliases (Zarr V2)

  • zfpy

Codec configuration Example - ZfpyCodecConfiguration:

Encode in fixed rate mode with 10.5 compressed bits per value
{
    "mode": 2,
    "rate": 10.5
}
Encode in fixed precision mode with 19 uncompressed bits per value
{
    "mode": 3,
    "precision": 19
}
Encode in fixed accuracy mode with a tolerance of 0.05
{
    "mode": 4,
    "tolerance": 0.05
}

Structs

ZfpyCodec
A zfpy codec implementation.
ZfpyCodecConfigurationNumcodecs
zfpy codec configuration parameters (numcodecs).

Enums

ZfpyCodecConfiguration
A wrapper to handle various versions of zfpy codec configuration parameters.