Module zstd

Module zstd 

Available on crate feature zstd only.
The zstd bytes to bytes codec.

Applies Zstd compression.

Compatible Implementations

This is expected to become a standardised extension.

Some implementations have compatibility issues due to how the content size is encoded:

Specification:

Codec name Aliases (Zarr V3)

  • zstd

Codec id Aliases (Zarr V2)

  • zstd

Codec configuration Example - ZstdCodecConfiguration:

{
    "level": 1,
    "checksum": true
}

Structs

ZstdCodec
A zstd codec implementation.
ZstdCodecConfigurationNumcodecs
zstd codec configuration parameters (numcodecs version 0.1).
ZstdCodecConfigurationV1
zstd codec configuration parameters (version 1.0).
ZstdCompressionLevel
A Zstd compression level. An integer from -131072 to 22 which controls the speed and level of compression (has no impact on decoding).

Enums

ZstdCodecConfiguration
A wrapper to handle various versions of zstd codec configuration parameters.