StorageTransformerChain

Struct StorageTransformerChain 

pub struct StorageTransformerChain(/* private fields */);
Configuration for a storage transformer chain.

Implementations

Source

impl StorageTransformerChain

pub fn new(storage_transformers: Vec<StorageTransformer>) -> Self

Create a storage transformer chain from a list of storage transformers.

pub fn from_metadata( metadatas: &[MetadataV3], path: &NodePath, ) -> Result<Self, PluginCreateError>

Create a storage transformer chain from configurations.

Errors

Returns PluginCreateError if there is a configuration issue or attempt to create an unregistered storage transformer.

pub fn create_metadatas(&self) -> Vec<MetadataV3>

Create storage transformer chain metadata.

Panics

Panics if any storage transformer does not have a V3 name.

Source

impl StorageTransformerChain

pub fn create_readable_transformer( &self, storage: ReadableStorage, ) -> Result<ReadableStorage, StorageError>

Create a readable storage transformer.

Errors

Returns an error if creation fails.

pub fn create_writable_transformer( &self, storage: WritableStorage, ) -> Result<WritableStorage, StorageError>

Create a writable storage transformer.

Errors

Returns an error if creation fails.

pub fn create_readable_writable_transformer( &self, storage: ReadableWritableStorage, ) -> Result<ReadableWritableStorage, StorageError>

Create a readable and writable storage transformer.

Errors

Returns an error if creation fails.

pub fn create_listable_transformer( &self, storage: ListableStorage, ) -> Result<ListableStorage, StorageError>

Create a listable storage transformer.

Errors

Returns an error if creation fails.

pub async fn create_async_readable_transformer( &self, storage: AsyncReadableStorage, ) -> Result<AsyncReadableStorage, StorageError>

Available on crate feature async only.

Create an asynchronous readable storage transformer.

Errors

Returns an error if creation fails.

pub async fn create_async_writable_transformer( &self, storage: AsyncWritableStorage, ) -> Result<AsyncWritableStorage, StorageError>

Available on crate feature async only.

Create an asynchronous writable storage transformer.

Errors

Returns an error if creation fails.

pub async fn create_async_readable_writable_transformer( &self, storage: AsyncReadableWritableStorage, ) -> Result<AsyncReadableWritableStorage, StorageError>

Available on crate feature async only.

Create an asynchronous redable and writable storage transformer.

Errors

Returns an error if creation fails.

pub async fn create_async_listable_transformer( &self, storage: AsyncListableStorage, ) -> Result<AsyncListableStorage, StorageError>

Available on crate feature async only.

Create an asynchronous listable storage transformer.

Errors

Returns an error if creation fails.