pub struct ArrayBytesOptional<'a> { /* private fields */ }Expand description
Optional array bytes composed of data and a validity mask.
The mask is 1 byte per element where 0 = invalid/missing, non-zero = valid/present. The mask length is validated at construction to ensure it matches the number of elements.
Equality Semantics
Two ArrayBytesOptional values are considered equal if:
- Their masks are equal
- Their valid (non-null) elements are equal
Invalid (null) element bytes are not compared, since they may differ after round-trip encoding/decoding (e.g., fill value vs original bytes).
Implementations
Sourceimpl<'a> ArrayBytesOptional<'a>
impl<'a> ArrayBytesOptional<'a>
Sourcepub fn new(
data: impl Into<Box<ArrayBytes<'a>>>,
mask: impl Into<Cow<'a, [u8]>>,
) -> ArrayBytesOptional<'a>
pub fn new( data: impl Into<Box<ArrayBytes<'a>>>, mask: impl Into<Cow<'a, [u8]>>, ) -> ArrayBytesOptional<'a>
Create a new ArrayBytesOptional with validation.
Sourcepub fn data(&self) -> &ArrayBytes<'a>
pub fn data(&self) -> &ArrayBytes<'a>
Get the underlying data.
Sourcepub fn num_elements(&self) -> usize
pub fn num_elements(&self) -> usize
Get the number of elements.
Sourcepub fn into_parts(self) -> (Box<ArrayBytes<'a>>, Cow<'a, [u8]>)
pub fn into_parts(self) -> (Box<ArrayBytes<'a>>, Cow<'a, [u8]>)
Consume self and return the data and mask.
Sourcepub fn into_owned(self) -> ArrayBytesOptional<'static>
pub fn into_owned(self) -> ArrayBytesOptional<'static>
Convert into owned ArrayBytesOptional<'static>.