pub struct RecommendedConcurrency { /* private fields */ }Expand description
The recommended concurrency of a codec includes the most efficient and maximum recommended concurrency.
Consider a chain that does slow decoding first on a single thread, but subsequent codecs can run on multiple threads. In this case, recommended concurrency is best expressed by two numbers:
- the efficient concurrency, equal to the minimum of codecs
- the maximum concurrency, equal to the maximum of codecs
Implementations
Sourceimpl RecommendedConcurrency
impl RecommendedConcurrency
Sourcepub fn new(range: impl RangeBounds<usize>) -> RecommendedConcurrency
pub fn new(range: impl RangeBounds<usize>) -> RecommendedConcurrency
Create a new recommended concurrency struct with an explicit concurrency range and preferred concurrency.
A minimum concurrency of zero is interpreted as a minimum concurrency of one.
Sourcepub fn new_minimum(minimum: usize) -> RecommendedConcurrency
pub fn new_minimum(minimum: usize) -> RecommendedConcurrency
Create a new recommended concurrency struct with a specified minimum concurrency and unbounded maximum concurrency.
Sourcepub fn new_maximum(maximum: usize) -> RecommendedConcurrency
pub fn new_maximum(maximum: usize) -> RecommendedConcurrency
Create a new recommended concurrency struct with a specified maximum concurrency.