Module node

Module node 

Zarr nodes.

A node in a Zarr hierarchy represents either an Array or a Group.

A Node has an associated NodePath, NodeMetadata, and children.

The Node::hierarchy_tree function can be used to create a string representation of a the hierarchy below a node.

Structs

Node
A Zarr hierarchy node.
NodeName
A Zarr hierarchy node name.
NodeNameError
An invalid node name.
NodePath
A Zarr hierarchy node path.
NodePathError
An invalid node path.

Enums

NodeCreateError
A node creation error.
NodeMetadata
Zarr node metadata (ArrayMetadata or GroupMetadata).

Functions

async_get_child_nodesasync
Asynchronously get the child nodes.
async_get_child_nodes_optasync
Asynchronously get the child nodes.
async_node_existsasync
Asynchronously check if a node exists.
async_node_exists_listableasync
Asynchronously check if a node exists.
data_key
Return the data key given a node path and a chunk_key of an array.
get_child_nodes
Get the child nodes.
get_child_nodes_opt
Get the child nodes.
meta_key
Return the Zarr V3 metadata key (zarr.json) given a node path.
meta_key_v3
Return the Zarr V3 metadata key (zarr.json) given a node path.
meta_key_v2_array
Return the Zarr V2 array metadata key (.zarray) given a node path.
meta_key_v2_attributes
Return the Zarr V2 user-defined attributes key (.zattrs) given a node path.
meta_key_v2_group
Return the Zarr V2 group metadata key (.zgroup) given a node path.
node_exists
Check if a node exists.
node_exists_listable
Check if a node exists.