Enum zarrs::storage::StorageError
source · pub enum StorageError {
ReadOnly,
IOError(Error),
InvalidJSON(Error),
StorePrefixError(StorePrefixError),
InvalidStoreKey(StoreKeyError),
NodePathError(NodePathError),
NodeNameError(NodeNameError),
InvalidByteRangeError(InvalidByteRangeError),
Unsupported(String),
UnknownKeySize(StoreKey),
Other(String),
}Expand description
A storage error.
Variants
ReadOnly
A write operation was attempted on a read only store.
IOError(Error)
An IO error.
InvalidJSON(Error)
An error serializing or deserializing JSON.
StorePrefixError(StorePrefixError)
An invalid store prefix.
InvalidStoreKey(StoreKeyError)
An invalid store key.
NodePathError(NodePathError)
An invalid node path.
NodeNameError(NodeNameError)
An invalid node name.
InvalidByteRangeError(InvalidByteRangeError)
An invalid byte range.
Unsupported(String)
The requested method is not supported.
UnknownKeySize(StoreKey)
Unknown key size where the key size must be known.
Other(String)
Any other error.