pub struct V2ChunkKeyEncoding { /* private fields */ }Expand description
A v2 chunk key encoding.
The identifier for chunk with at least one dimension is formed by concatenating for each dimension:
- the ASCII decimal string representation of the chunk index within that dimension, followed by
- the separator character, except that it is omitted for the last dimension.
See https://zarr-specs.readthedocs.io/en/latest/v3/chunk-key-encodings/v2/index.html.
Implementations
Sourceimpl V2ChunkKeyEncoding
impl V2ChunkKeyEncoding
Sourcepub const fn new(separator: ChunkKeySeparator) -> Self
pub const fn new(separator: ChunkKeySeparator) -> Self
Create a new v2 chunk key encoding with separator separator.