Available on crate feature
bitround only.Expand description
The bitround array to array codec.
Round the mantissa of floating point data types to the specified number of bits. Rounds integers from the most significant set bit. Bit rounding leaves an array more amenable to compression.
This codec requires the bitround feature, which is disabled by default.
Compatible Implementations
This codec is fully compatible with the numcodecs.bitround codec in zarr-python.
However, it supports additional data types not supported by that implementation.
Specification
- https://github.com/zarr-developers/zarr-extensions/blob/main/codecs/bitround/README.md
- https://codec.zarrs.dev/array_to_array/bitround
Codec name Aliases (Zarr V3)
bitroundnumcodecs.bitroundhttps://codec.zarrs.dev/array_to_array/bitround
Codec id Aliases (Zarr V2)
bitround
Codec configuration Example - BitroundCodecConfiguration:
{
"keepbits": 10
}Structs
- Bitround
Codec - A
bitroundcodec implementation. - Bitround
Codec Configuration V1 bitroundcodec configuration parameters (version 1.0 draft).- Bitround
Data Type Plugin - Plugin for registering
bitroundcodec support for a data type.
Enums
- Bitround
Codec Configuration - A wrapper to handle various versions of
bitroundcodec configuration parameters.
Traits
- Bitround
Data Type Ext - Extension trait for
DataTypeto accessBitroundDataTypeTraits. - Bitround
Data Type Traits - Traits for a data type supporting the
bitroundcodec.
Functions
- round_
bytes_ float16 - Helper to round 16-bit float values (fixed mantissa bits).
- round_
bytes_ float32 - Helper to round 32-bit float values (fixed mantissa bits).
- round_
bytes_ float64 - Helper to round 64-bit float values (fixed mantissa bits).
- round_
bytes_ int8 - Helper to round 8-bit integer values (from MSB).
- round_
bytes_ int16 - Helper to round 16-bit integer values (from MSB).
- round_
bytes_ int32 - Helper to round 32-bit integer values (from MSB).
- round_
bytes_ int64 - Helper to round 64-bit integer values (from MSB).