Available on crate feature
pcodec only.Expand description
The pcodec array to bytes codec (Experimental).
This codec is experimental and may be incompatible with other Zarr V3 implementations.
Pcodec (or Pco, pronounced “pico”) losslessly compresses and decompresses numerical sequences with high compression ratio and fast speed.
This codec requires the pcodec feature, which is disabled by default.
Compatible Implementations:
This codec is fully compatible with the numcodecs.pcodec codec in zarr-python.
Specification
Codec name Aliases (Zarr V3)
numcodecs.pcodechttps://codec.zarrs.dev/array_to_bytes/pcodec
Codec id Aliases (Zarr V2)
pcodec
Codec configuration Example - PcodecCodecConfiguration:
{
"level": 5,
"mode_spec": "auto",
"delta_spec": "auto",
"paging_spec": "equal_pages_up_to",
"delta_encoding_order": null,
"equal_pages_up_to": 262144
}Structs
- Pcodec
Codec - A
pcodeccodec implementation. - Pcodec
Codec Configuration V1 pcodeccodec configuration parameters (version 1.0 draft).- Pcodec
Compression Level - An integer from 0 to 12 controlling the compression level.
- Pcodec
Data Type Plugin - Plugin for registering
pcodeccodec support for a data type. - Pcodec
Delta Encoding Order - An integer from 0 to 7 controlling the delta encoding order.
Enums
- Pcodec
Codec Configuration - A wrapper to handle various versions of
pcodeccodec configuration parameters. - Pcodec
Element Type - The pcodec element type for dispatching to the pcodec library.
Traits
- Pcodec
Data Type Ext - Extension trait for
DataTypeto accessPcodecDataTypeTraits. - Pcodec
Data Type Traits - Traits for a data type supporting the
pcodeccodec.