Available on crate feature
zstd only.Expand description
The zstd bytes to bytes codec.
Applies Zstd compression.
Compatible Implementations
This is expected to become a standardised extension.
Some implementations have compatibility issues due to how the content size is encoded:
- https://github.com/zarr-developers/numcodecs/issues/424
- https://github.com/google/neuroglancer/issues/625
Specification:
- https://github.com/zarr-developers/zarr-extensions/tree/main/codecs/zstd
- https://github.com/zarr-developers/zarr-specs/pull/256
Codec name Aliases (Zarr V3)
zstd
Codec id Aliases (Zarr V2)
zstd
Codec configuration Example - ZstdCodecConfiguration:
{
"level": 1,
"checksum": true
}Structs
- Zstd
Codec - A
zstdcodec implementation. - Zstd
Codec Configuration Numcodecs zstdcodec configuration parameters (numcodecsversion 0.1).- Zstd
Codec Configuration V1 zstdcodec configuration parameters (version 1.0).- Zstd
Compression Level - A
Zstdcompression level. An integer from -131072 to 22 which controls the speed and level of compression (has no impact on decoding).
Enums
- Zstd
Codec Configuration - A wrapper to handle various versions of
zstdcodec configuration parameters.