Zarr nodes.
A node in a Zarr hierarchy represents either an Array or a Group.
A Node has an associated NodePath, NodeMetadata, and children.
The Node::hierarchy_tree function can be used to create a string representation of a the hierarchy below a node.
Structs
- Node
- A Zarr hierarchy node.
- Node
Name - A Zarr hierarchy node name.
- Node
Name Error - An invalid node name.
- Node
Path - A Zarr hierarchy node path.
- Node
Path Error - An invalid node path.
Enums
- Node
Create Error - A node creation error.
- Node
Metadata - Zarr node metadata (
ArrayMetadataorGroupMetadata).
Functions
- async_
get_ child_ nodes async - Asynchronously get the child nodes.
- async_
get_ child_ nodes_ opt async - Asynchronously get the child nodes.
- async_
node_ exists async - Asynchronously check if a node exists.
- async_
node_ exists_ listable async - Asynchronously check if a node exists.
- data_
key - Return the data key given a node path and a
chunk_keyof an array. - get_
child_ nodes - Get the child nodes.
- get_
child_ nodes_ opt - Get the child nodes.
- meta_
key - Return the Zarr V3 metadata key (zarr.json) given a node path.
- meta_
key_ v3 - Return the Zarr V3 metadata key (zarr.json) given a node path.
- meta_
key_ v2_ array - Return the Zarr V2 array metadata key (.zarray) given a node path.
- meta_
key_ v2_ attributes - Return the Zarr V2 user-defined attributes key (.zattrs) given a node path.
- meta_
key_ v2_ group - Return the Zarr V2 group metadata key (.zgroup) given a node path.
- node_
exists - Check if a node exists.
- node_
exists_ listable - Check if a node exists.